@@ -39,6 +39,7 @@
 #endif
 
 /* Application-specific */
+#include "version.h"
 #include "data.h"
 #include "whois.h"
 #include "utils.h"
@@ -458,6 +459,21 @@ char *guess_server(const char *s)
 	if (domcmp(s, tld_serv[i]))
 	    return strdup(tld_serv[i + 1]);
 
+    /* use the default server name for "new" gTLDs */
+    if (is_new_gtld(s)) {
+	char *server;
+	const char *p, *tld = NULL;
+
+	for (p = s; *p; p++)		/* look for the TLD */
+	    if (*p == '.')
+		tld = p;
+
+	server = malloc(strlen("whois.nic") + strlen(tld) + 1);
+	strcpy(server, "whois.nic");
+	strcat(server, tld);
+	return(server);
+    }
+
     /* no dot but hyphen */
     if (!strchr(s, '.')) {
 	/* search for strings at the start of the word */
@@ -534,6 +550,12 @@ char *queryformat(const char *server, const char *flags, const char *query)
 	    }
 	    break;
 	}
+
+    /* Use UTF-8 by default for "new" gTLDs */
+    if (!simple_recode_input_charset &&		/* was not in the database */
+	    !strchr(query, ' ') &&		/* and has no parameters */
+	    is_new_gtld(query))			/* and is a "new" gTLD: */
+	simple_recode_input_charset = "utf-8";	/* then try UTF-8 */
 #endif
 
 #ifdef HAVE_LIBIDN
@@ -956,6 +978,17 @@ int domcmp(const char *dom, const char *tld)
     return 0;
 }
 
+int is_new_gtld(const char *s)
+{
+    int i;
+
+    for (i = 0; new_gtlds[i]; i++)
+	if (domcmp(s, new_gtlds[i]))
+	    return 1;
+
+    return 0;
+}
+
 /*
  * Attempt to normalize a query by removing trailing dots and whitespace,
  * then convert the domain to punycode.
